# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging

from vitrage_tempest_plugin.tests.api.templates.base import BaseTemplateTest

LOG = logging.getLogger(__name__)

EXECUTE_MISTRAL_TEMPLATE = 'v2_execute_mistral.yaml'
EQUIVALENCE_TEMPLATE = 'v2_equivalence_templates.yaml'
DEFINITION_TEMPLATE = 'v2_definition_template.yaml'
NO_TYPE_TEMPLATE = 'v2_no_type_template.yaml'
WITH_PARAMS_TEMPLATE = 'v2_with_params.yaml'
WITH_DEFAULT_PARAMS_TEMPLATE = 'v2_with_default_params.yaml'

FAILED_TO_RESOLVE_PARAM = 'Failed to resolve parameter'
ERROR_STATUS = 'ERROR'
LOADING_STATUS = 'LOADING'
TEMPLATE_VALIDATION_OK = 'Template validation is OK'


class TestTemplatesApis(BaseTemplateTest):

    def _validate_no_type_templates(self, template_name):
        path = self.TEST_PATH + template_name
        validation = self.vitrage_client.template.validate(path=path)
        self._assert_validate_result(
            validation, path, negative=True, status_code=66)

    def _validate_standard_template(self, template_name):
        path = self.TEST_PATH + template_name
        validation = self.vitrage_client.template.validate(path=path)
        self._assert_validate_result(validation, path)

    def _validate_definition_template(self, template_name):
        path = self.TEST_PATH + template_name
        validation = self.vitrage_client.template.validate(path=path)
        self._assert_validate_result(validation, path)

    def _validate_with_missing_parameters(self, template_name):
        path = self.TEST_PATH + template_name
        validation = self.vitrage_client.template.validate(path=path)
        self._assert_validate_result(
            validation, path, negative=True, status_code=163)

    def _validate_with_missing_parameter(self, template_name):
        path = self.TEST_PATH + template_name
        params = {'template_name': 'My template 1',
                  'new_state': 'SUBOPTIMAL'}
        validation = \
            self.vitrage_client.template.validate(path=path, params=params)
        self._assert_validate_result(
            validation, path, negative=True, status_code=163)

    def _validate_with_parameters(self, template_name):
        path = self.TEST_PATH + template_name
        params = {'template_name': 'My template 1',
                  'alarm_type': 'Monitor1',
                  'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}
        validation = \
            self.vitrage_client.template.validate(path=path, params=params)
        self._assert_validate_result(validation, path)

    def _validate_with_default_parameters(self, template_name):
        path = self.TEST_PATH + template_name
        validation = \
            self.vitrage_client.template.validate(path=path, params=None)
        self._assert_validate_result(validation, path)

    def _add_with_missing_parameters(self, template_name):
        path = self.TEST_PATH + template_name
        result = self.vitrage_client.template.add(path=path)
        self._assert_add_result(result, ERROR_STATUS, FAILED_TO_RESOLVE_PARAM)

    def _add_with_missing_parameter(self, template_name):
        path = self.TEST_PATH + template_name
        params = {'template_name': 'My template 1',
                  'new_state': 'SUBOPTIMAL'}
        result = self.vitrage_client.template.add(path=path, params=params)
        self._assert_add_result(result, ERROR_STATUS, FAILED_TO_RESOLVE_PARAM)

    def _add_with_parameters(self, template_name):
        path = self.TEST_PATH + template_name
        params = {'template_name': 'My template 1',
                  'alarm_type': 'Monitor1',
                  'alarm_name': 'My alarm',
                  'new_state': 'SUBOPTIMAL'}
        result = self.vitrage_client.template.add(path=path, params=params)
        self._assert_add_result(result, LOADING_STATUS, TEMPLATE_VALIDATION_OK)

    def _add_with_default_parameters(self, template_name):
        path = self.TEST_PATH + template_name
        result = self.vitrage_client.template.add(path=path, params=None)
        self._assert_add_result(result, LOADING_STATUS, TEMPLATE_VALIDATION_OK)

    def _add_by_string(self, template_str):
        result = self.vitrage_client.template.add(template_str=template_str)
        self._assert_add_result(result, LOADING_STATUS, TEMPLATE_VALIDATION_OK)


TEMPLATE_STRING = """
metadata:
 name: host_down_scenarios
 description: scenarios triggered by Doctor monitor 'compute.host.down' alarm
 version: 2
 type: standard
definitions:
 entities:
  - entity:
     category: ALARM
     name: compute.host.down
     template_id: host_down_alarm
  - entity:
     category: RESOURCE
     type: nova.host
     template_id: host
 relationships:
  - relationship:
     source: host_down_alarm
     relationship_type: on
     target: host
     template_id : host_down_alarm_on_host
scenarios:
 - scenario:
    condition: host_down_alarm_on_host
    actions:
     - action:
        action_type: set_state
        action_target:
         target: host
        properties:
         state: ERROR
"""


class TestTemplatesV2(TestTemplatesApis):

    def test_templates_validate_no_type_templates(self):
        self._validate_no_type_templates(NO_TYPE_TEMPLATE)

    def test_templates_validate_standard_template(self):
        self._validate_standard_template(EXECUTE_MISTRAL_TEMPLATE)

    def test_templates_validate_definition_template(self):
        self._validate_definition_template(DEFINITION_TEMPLATE)

    def test_template_validate_with_missing_parameters(self):
        self._validate_with_missing_parameters(WITH_PARAMS_TEMPLATE)

    def test_template_validate_with_missing_parameter(self):
        self._validate_with_missing_parameter(WITH_PARAMS_TEMPLATE)

    def test_template_validate_with_parameters(self):
        self._validate_with_parameters(WITH_PARAMS_TEMPLATE)

    def test_template_validate_with_default_parameters(self):
        self._validate_with_default_parameters(WITH_DEFAULT_PARAMS_TEMPLATE)

    def test_template_add_with_missing_parameters(self):
        self._add_with_missing_parameters(WITH_PARAMS_TEMPLATE)

    def test_template_add_with_missing_parameter(self):
        self._add_with_missing_parameter(WITH_PARAMS_TEMPLATE)

    def test_template_add_with_parameters(self):
        self._add_with_parameters(WITH_PARAMS_TEMPLATE)

    def test_template_add_with_default_parameters(self):
        self._add_with_default_parameters(WITH_DEFAULT_PARAMS_TEMPLATE)

    def test_template_add_by_string(self):
        self._add_by_string(TEMPLATE_STRING)



#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import socket

from testtools import matchers

from vitrage_tempest_plugin.tests.base import BaseVitrageTempest


SERVICES = {
    'ApiWorker',
    'EvaluatorWorker',
    'MachineLearningService',
    'PersistorService',
    'SnmpParsingService',
    'vitrageuWSGI',
    'VitrageNotifierService'
}


class ServiceTest(BaseVitrageTempest):
    def test_service_list(self):
        services = self.vitrage_client.service.list()

        self.check_all_equal(services, SERVICES)
        self.check_all_hosted(services, socket.gethostname())
        self.check_different_process_ids_for(services)

    def check_all_equal(self, services, expected_svc_names):
        names = {service['name'].split('_')[0] for service in services}
        self.assert_set_equal(expected_svc_names, names)

    def check_all_hosted(self, services, hostname):
        hostnames = {service['hostname'] for service in services}
        self.assert_set_equal({hostname}, hostnames)

    def check_different_process_ids_for(self, services):
        num_services = len(services)
        process_ids = {service['process'] for service in services}
        self.assertThat(process_ids, matchers.HasLength(num_services))



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import socket
import time

from oslo_log import log as logging
from vitrage_tempest_plugin.tests.base import BaseVitrageTempest
from vitrage_tempest_plugin.tests.common.general_utils \
    import tempest_resources_dir
from vitrage_tempest_plugin.tests import utils

LOG = logging.getLogger(__name__)


class TestStatic(BaseVitrageTempest):
    NUM_SWITCH = 1
    NUM_NIC = 1

    def setUp(self):
        super(TestStatic, self).setUp()
        self._create_switches()

    def tearDown(self):
        super(TestStatic, self).tearDown()
        self._delete_switches()

    @utils.tempest_logger
    def test_switches(self):
        # Calculate expected results
        api_graph = self.vitrage_client.topology.get(all_tenants=True)
        graph = self._create_graph_from_graph_dictionary(api_graph)
        entities = self._entities_validation_data(
            host_entities=1,
            host_edges=1,
            switch_entities=self.NUM_SWITCH,
            switch_edges=1,
            nic_entities=self.NUM_NIC,
            nic_edges=1)
        num_entities = self.num_default_entities + self.NUM_SWITCH + \
            self.NUM_NIC + self.num_default_networks + \
            self.num_default_ports
        num_edges = self.num_default_edges + self.NUM_SWITCH + \
            self.num_default_ports

        # Test Assertions
        self._validate_graph_correctness(graph,
                                         num_entities,
                                         num_edges,
                                         entities)

    @staticmethod
    def _create_switches():
        hostname = socket.gethostname()

        # template file
        file_path = \
            tempest_resources_dir() + '/static/static_configuration.yaml'
        with open(file_path, 'r') as f:
            template_data = f.read()
        template_data = template_data.replace('tmp-devstack', hostname)

        # new file
        new_file = open(
            '/etc/vitrage/static_datasources/static_configuration.yaml', 'w')
        new_file.write(template_data)
        new_file.close()

        time.sleep(35)

    @staticmethod
    def _delete_switches():
        path = '/etc/vitrage/static_datasources/static_configuration.yaml'
        if os.path.exists(path):
            os.remove(path)

        time.sleep(35)



# Copyright 2017 Nokia
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from testtools import matchers

from vitrage_tempest_plugin.tests.base import BaseVitrageTempest
from vitrageclient.exceptions import ClientException

LOG = logging.getLogger(__name__)

URL = 'url'
REGEX_FILTER = 'regex_filter'
HEADERS = 'headers'
HEADERS_PROPS = '{"content": "application/json"}'
REGEX_PROPS = '{"name": "e2e.*"}'


class TestWebhook(BaseVitrageTempest):
    """Webhook test class for Vitrage API tests."""

    @classmethod
    def setUpClass(cls):
        super(TestWebhook, cls).setUpClass()
        cls.pre_test_webhook_count = \
            len(cls.vitrage_client.webhook.list())

    def test_add_webhook(self):

        webhooks = self.vitrage_client.webhook.list()
        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count),
                        'Amount of webhooks should be '
                        'the same as before the test')

        created_webhook = self.vitrage_client.webhook.add(
            url="https://www.test.com",
            regex_filter=REGEX_PROPS,
            headers=HEADERS_PROPS
        )

        self.assertIsNone(created_webhook.get('ERROR'), 'webhook not '
                                                        'created')
        self.assertEqual(created_webhook[HEADERS],
                         HEADERS_PROPS,
                         'headers not created correctly')
        self.assertEqual(created_webhook[REGEX_FILTER],
                         REGEX_PROPS,
                         'regex not created correctly')
        self.assertEqual(created_webhook[URL],
                         "https://www.test.com",
                         'URL not created correctly')

        webhooks = self.vitrage_client.webhook.list()

        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count + 1))
        self.vitrage_client.webhook.delete(
            created_webhook['id'])

    def test_delete_webhook(self):
        webhooks = self.vitrage_client.webhook.list()
        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count),
                        'Amount of webhooks should '
                        'be the same as before the test')

        created_webhook = self.vitrage_client.webhook.add(
            url="https://www.test.com",
            regex_filter=REGEX_PROPS,
            headers=HEADERS_PROPS
        )

        created_webhook = self.vitrage_client.webhook.delete(
            id=created_webhook['id'])
        self.assertIsNotNone(created_webhook.get('SUCCESS'),
                             'failed to delete')
        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count),
                        'No webhooks should exist after deletion')

    def test_delete_non_existing_webhook(self):
        self.assertRaises(ClientException,
                          self.vitrage_client.webhook.delete,
                          'non existent')

    def test_list_webhook(self):

        webhooks = self.vitrage_client.webhook.list()
        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count),
                        'Amount of webhooks should be '
                        'the same as before the test')

        created_webhook = self.vitrage_client.webhook.add(
            url="https://www.test.com",
            regex_filter=REGEX_PROPS,
            headers=HEADERS_PROPS
        )

        webhooks = self.vitrage_client.webhook.list()
        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count + 1))
        self.assertEqual(created_webhook[HEADERS], webhooks[0][HEADERS])
        self.assertEqual(created_webhook['id'], webhooks[0]['id'])
        self.assertEqual(created_webhook[REGEX_FILTER],
                         webhooks[0][REGEX_FILTER])

        self.vitrage_client.webhook.delete(created_webhook['id'])

    def test_show_webhook(self):
        webhooks = self.vitrage_client.webhook.list()
        self.assertThat(webhooks,
                        matchers.HasLength(self.pre_test_webhook_count),
                        'Amount of webhooks should be '
                        'the same as before the test')

        created_webhook = self.vitrage_client.webhook.add(
            url="https://www.test.com",
            regex_filter=REGEX_PROPS,
            headers=HEADERS_PROPS
        )

        show_webhook = self.vitrage_client.webhook.show(created_webhook['id'])

        self.assertIsNotNone(show_webhook, 'webhook not listed')
        self.assertEqual(created_webhook[HEADERS],
                         show_webhook[HEADERS],
                         'headers mismatch')
        self.assertEqual(created_webhook[REGEX_FILTER],
                         show_webhook[REGEX_FILTER],
                         'regex mismatch')
        self.assertEqual(created_webhook[URL],
                         show_webhook[URL],
                         'URL mismatch')

        self.vitrage_client.webhook.delete(created_webhook['id'])



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from concurrent import futures
from threading import Thread
import time

from oslo_log import log as logging
from tempest import config


from vitrage_tempest_plugin.tests.common.constants import VertexProperties
from vitrage_tempest_plugin.tests.common import general_utils as g_utils
from vitrage_tempest_plugin.tests.common import vitrage_utils as v_utils
from vitrage_tempest_plugin.tests.e2e.test_actions_base import TestActionsBase
from vitrage_tempest_plugin.tests import utils

CONF = config.CONF
LOG = logging.getLogger(__name__)

DEDUCED_1 = 'mock_datasource.3rd_degree_scenarios.deduced.alarm1'
DEDUCED_2 = 'mock_datasource.3rd_degree_scenarios.deduced.alarm2'
TEMPLATE_NAME = 'mock_datasource_3rd_degree_scenarios.yaml'
SLEEP = 100
MAX_FAIL_OVER_TIME = 20


class TestLongProcessing(TestActionsBase):

    def tearDown(self):
        super(TestLongProcessing, self).tearDown()
        if v_utils.get_first_template(name=TEMPLATE_NAME):
            v_utils.delete_template(name=TEMPLATE_NAME)
            time.sleep(SLEEP)

    @classmethod
    def setUpClass(cls):
        super(TestLongProcessing, cls).setUpClass()
        logger = logging.getLogger('vitrageclient.v1.client').logger
        logger.setLevel(logging.INFO)
        if v_utils.get_first_template(name=TEMPLATE_NAME):
            v_utils.delete_template(name=TEMPLATE_NAME)
            time.sleep(SLEEP)

    @utils.tempest_logger
    def test_high_availability_events(self):
        """The purpose of the test is to check that events are stored

        That is, during different stages in vitrage-graph lifetime:
        before graph read from db (during init)
        after graph read from db (during init)
        during get_all
        after get_all
        """
        try:
            # adding a template just to create more load (to slow things down)
            v_utils.add_template(TEMPLATE_NAME)
            time.sleep(SLEEP)
            self.keep_sending_events = True
            self.num_of_sent_events = 0

            doctor_events_thread = self._async_doctor_events()
            time.sleep(10)
            v_utils.stop_graph()
            time.sleep(10)
            v_utils.restart_graph()
            time.sleep(MAX_FAIL_OVER_TIME)
            v_utils.delete_template(name=TEMPLATE_NAME)

            # sleep to allow get_all to start and finish at least once:
            time.sleep(4 * CONF.root_cause_analysis_service.snapshots_interval)

            v_utils.restart_graph()
            self.keep_sending_events = False
            time.sleep(MAX_FAIL_OVER_TIME)
            doctor_events_thread.join(timeout=10)

            alarm_count = self.vitrage_client.alarm.count(all_tenants=True)
            self.assertTrue(self.num_of_sent_events > 0,
                            'Test did not create events')
            self.assertAlmostEqual(
                self.num_of_sent_events,
                alarm_count['CRITICAL'],
                msg='CRITICAL doctor events expected',
                delta=1)
        finally:
            self._remove_doctor_events()

    @utils.tempest_logger
    def test_db_init(self):
        v_utils.add_template(TEMPLATE_NAME)
        time.sleep(SLEEP)

        # 1. check template works well
        self._check_template_instance_3rd_degree_scenarios()

        # 2. check fast fail-over - start from database
        topo1 = self.vitrage_client.topology.get(all_tenants=True)
        v_utils.restart_graph()
        time.sleep(MAX_FAIL_OVER_TIME)
        for i in range(5):
            self._check_template_instance_3rd_degree_scenarios()
            topo2 = self.vitrage_client.topology.get(all_tenants=True)
            self._assert_graph_equal(
                topo1, topo2, 'comparing graph items iteration %s' % i)
            time.sleep(CONF.root_cause_analysis_service.snapshots_interval)

        v_utils.delete_template(name=TEMPLATE_NAME)
        time.sleep(SLEEP)
        self._check_template_instance_3rd_degree_scenarios_deleted()

    def _check_template_instance_3rd_degree_scenarios(self):

        alarm_count = self.vitrage_client.alarm.count(all_tenants=True)
        self.assertEqual(
            CONF.root_cause_analysis_service.instances_per_host,
            alarm_count['SEVERE'],
            'Each instance should have one SEVERE deduced alarm')
        self.assertEqual(
            CONF.root_cause_analysis_service.instances_per_host,
            alarm_count['CRITICAL'],
            'Each instance should have one CRITICAL deduced alarm')

        expected_rca = [{VertexProperties.VITRAGE_TYPE: 'zabbix'}] * CONF.\
            root_cause_analysis_service.zabbix_alarms_per_host
        expected_rca.extend([{'name': DEDUCED_1}, {'name': DEDUCED_2}])

        def check_rca(alarm):
            rca = self.vitrage_client.rca.get(alarm['vitrage_id'],
                                              all_tenants=True)
            try:
                self._check_rca(rca, expected_rca, alarm)
                return True
            except Exception:
                LOG.exception('check_rca failed')
                return False

        # 10 threads calling rca api
        alarms = self.vitrage_client.alarm.list(all_tenants=True,
                                                vitrage_id='all')
        deduced_alarms = g_utils.all_matches(
            alarms, vitrage_type='vitrage', name=DEDUCED_2)
        workers = futures.ThreadPoolExecutor(max_workers=10)
        workers_result = [r for r in workers.map(check_rca,
                                                 deduced_alarms)]
        self.assertTrue(all(workers_result))

    def _check_template_instance_3rd_degree_scenarios_deleted(self):
        alarm_count = self.vitrage_client.alarm.count(all_tenants=True)
        self.assertEqual(
            0,
            alarm_count['SEVERE'],
            'found SEVERE deduced alarms after template delete')
        self.assertEqual(
            0,
            alarm_count['CRITICAL'],
            'found CRITICAL deduced alarms after template delete')

    def _async_doctor_events(self, spacing=1):

        def do_create():
            while self.keep_sending_events:
                try:
                    v_utils.generate_fake_host_alarm(
                        'nova.host-0-nova.zone-0-openstack.cluster-0',
                        'test_high_availability_events' +
                        str(self.num_of_sent_events))
                    self.num_of_sent_events += 1
                    time.sleep(spacing)
                except Exception:
                    time.sleep(spacing)
                    continue

        t = Thread(target=do_create)
        t.setDaemon(True)
        t.start()
        return t

    def _remove_doctor_events(self):

        for i in range(self.num_of_sent_events):
            try:
                v_utils.generate_fake_host_alarm(
                    'nova.host-0-nova.zone-0-openstack.cluster-0',
                    'test_high_availability_events %s' % i,
                    enabled=False)
            except Exception:
                continue



# Copyright 2016 Nokia
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time


from vitrage_tempest_plugin.tests.base import BaseVitrageTempest
from vitrage_tempest_plugin.tests.base import IsNotEmpty
from vitrage_tempest_plugin.tests.base import LOG
from vitrage_tempest_plugin.tests.common import cinder_utils
from vitrage_tempest_plugin.tests.common import nova_utils


class BaseTopologyTest(BaseVitrageTempest):
    """Topology test class for Vitrage API tests."""

    def tearDown(self):
        super(BaseTopologyTest, self).tearDown()
        self._rollback_to_default()

    def _rollback_to_default(self):
        self._delete_entities()
        api_graph = self.vitrage_client.topology.get(
            root=None,
            all_tenants=True)
        graph = self._create_graph_from_graph_dictionary(api_graph)
        entities = self._entities_validation_data()
        num_default_entities = self.num_default_entities + \
            self.num_default_networks + self.num_default_ports
        num_default_edges = self.num_default_edges + self.num_default_ports
        try:
            self._validate_graph_correctness(graph,
                                             num_default_entities,
                                             num_default_edges,
                                             entities)
        except AssertionError as e:
            LOG.error(e)

    def _create_entities(self, num_instances, num_volumes=0, end_sleep=3):
        resources = nova_utils.create_instances(num_instances)

        self.assertThat(resources, IsNotEmpty(),
                        'The instances list is empty')
        if num_volumes > 0:
            cinder_utils.create_volume_and_attach('volume-1', 1,
                                                  resources[0].id,
                                                  '/tmp/vda')

        # waiting until all the entities creation were processed by the
        # entity graph processor
        time.sleep(end_sleep)

    @staticmethod
    def _delete_entities():
        cinder_utils.delete_all_volumes()
        nova_utils.delete_all_instances()

        # waiting until all the entities deletion were processed by the
        # entity graph processor
        time.sleep(2)

    @staticmethod
    def _graph_query():
        return '{"and": [{"==": {"vitrage_category": "RESOURCE"}},' \
               '{"==": {"vitrage_is_deleted": false}},' \
               '{"==": {"vitrage_is_placeholder": false}},' \
               '{"or": [{"==": {"vitrage_type": "openstack.cluster"}},' \
               '{"==": {"vitrage_type": "nova.instance"}},' \
               '{"==": {"vitrage_type": "nova.host"}},' \
               '{"==": {"vitrage_type": "nova.zone"}}]}]}'

    @staticmethod
    def _tree_query():
        return '{"and": [{"==": {"vitrage_category": "RESOURCE"}},' \
               '{"==": {"vitrage_is_deleted": false}},' \
               '{"==": {"vitrage_is_placeholder": false}},' \
               '{"or": [{"==": {"vitrage_type": "openstack.cluster"}},' \
               '{"==": {"vitrage_type": "nova.host"}},' \
               '{"==": {"vitrage_type": "nova.zone"}}]}]}'

    @staticmethod
    def _graph_no_match_query():
        return '{"and": [{"==": {"vitrage_category": "test"}},' \
               '{"==": {"vitrage_is_deleted": false}},' \
               '{"==": {"vitrage_is_placeholder": false}},' \
               '{"or": [{"==": {"vitrage_type": "openstack.cluster"}},' \
               '{"==": {"vitrage_type": "nova.instance"}},' \
               '{"==": {"vitrage_type": "nova.host"}},' \
               '{"==": {"vitrage_type": "nova.zone"}}]}]}'

    @staticmethod
    def _tree_no_match_query():
        return '{"and": [{"==": {"vitrage_category": "test"}},' \
               '{"==": {"vitrage_is_deleted": false}},' \
               '{"==": {"vitrage_is_placeholder": false}},' \
               '{"or": [{"==": {"vitrage_type": "openstack.cluster"}},' \
               '{"==": {"vitrage_type": "nova.host"}},' \
               '{"==": {"vitrage_type": "nova.zone"}}]}]}'



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time

from oslo_log import log as logging

from vitrage_tempest_plugin.tests.common import general_utils as g_utils
from vitrage_tempest_plugin.tests.common import glance_utils
from vitrage_tempest_plugin.tests.common import neutron_utils
from vitrage_tempest_plugin.tests.common.tempest_clients import TempestClients
from vitrage_tempest_plugin.tests.utils import wait_for_status

LOG = logging.getLogger(__name__)


def create_instances(num_instances=1, set_public_network=True, name='vm'):
    nics = []
    flavor = get_first_flavor()
    image = glance_utils.get_first_image()
    if set_public_network:
        public_net = neutron_utils.get_neutron_network('public')
        if public_net:
            nics = [{'net-id': public_net['id']}]

    for i in range(3):
        success, resources = _create_instances(flavor, image, name, nics,
                                               num_instances)
        time.sleep(2)
        if success:
            return resources
        if not success:
            LOG.warning("create instance failed, delete and retry %s",
                        resources)
            delete_created_instances(resources)
            time.sleep(10)
    raise AssertionError("Unable to create vms, retries failed")


def _create_instances(flavor, image, name, nics, num_instances):
    LOG.info("create instance with flavor=%s image=%s" % (flavor, image))
    resources = [TempestClients.nova().servers.create(
        name='%s-%s' % (name, index),
        flavor=flavor,
        image=image,
        nics=nics) for index in range(num_instances)]
    success = wait_for_status(
        30, check_new_instances, ids=[instance.id for instance in resources])
    return success, resources


def delete_created_instances(instances):
    if not instances:
        return
    for instance in instances:
        delete_all_instances(id=instance.id)


def delete_all_instances(**kwargs):
    instances = TempestClients.nova().servers.list()
    instances_to_delete = g_utils.all_matches(instances, **kwargs)
    for item in instances_to_delete:
        try:
            TempestClients.nova().servers.force_delete(item)
        except Exception:
            LOG.exception('Failed to force delete instance %s', item.id)
    wait_for_status(
        30,
        check_deleted_instances,
        ids=[instance.id for instance in instances_to_delete])
    time.sleep(2)


def get_first_flavor():
    return TempestClients.nova().flavors.list(sort_key='memory_mb')[0]


def check_deleted_instances(ids):
    servers = TempestClients.nova().servers.list()
    for s in servers:
        if s.id in ids:
            return False
    return True


def check_new_instances(ids):
    servers = TempestClients.nova().servers.list()
    for _id in ids:
        ok = False
        for s in servers:
            if _id == s.id and s.status.lower() == 'active':
                ok = True
                break
        if not ok:
            return False
    return True



# Copyright 2016 Nokia
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneauth1 import loading as ka_loading
from keystoneauth1 import session as ka_session
from keystoneclient.v3 import client as ks_client_v3
from neutronclient.v2_0 import client as neutron_client
from oslo_utils import importutils as utils
from tempest.common import credentials_factory as common_creds
from tempest import config
from vitrageclient import client as vc

CONF = config.CONF

_client_modules = {
    'aodh': 'aodhclient.client',
    'nova': 'novaclient.client',
    'cinder': 'cinderclient.client',
    'glance': 'glanceclient.client',
    'neutron': 'neutronclient.v2_0.client',
    'heat': 'heatclient.client',
    'mistral': 'mistralclient.api.v2.client',
    'gnocchi': 'gnocchiclient.v1.client',
    'trove': 'troveclient.v1.client'
}


def driver_module(driver):
    mod_name = _client_modules[driver]
    module = utils.import_module(mod_name)
    return module


class TempestClients(object):
    @classmethod
    def class_init(cls, creds):
        cls.creds = creds
        cls._vitrage = None
        cls._nova = None
        cls._cinder = None
        cls._glance = None
        cls._neutron = None
        cls._heat = None
        cls._mistral = None
        cls._aodh = None
        cls._keystone = None
        cls._gnocchi = None

    @classmethod
    def vitrage(cls):
        """vitrage client

        :rtype: vitrageclient.v1.client.Client
        """
        if not cls._vitrage:
            cls._vitrage = vc.Client(
                '1', session=cls._get_session_for_admin())
        return cls._vitrage

    @classmethod
    def vitrage_client_for_user(cls):
        """vitrage client for a specific user and tenant

        :rtype: vitrageclient.v1.client.Client
        """
        session = cls._get_session_for_user()
        return vc.Client('1', session=session)

    @classmethod
    def neutron_client_for_user(cls):
        session = cls._get_session_for_user()
        return neutron_client.Client(session=session)

    @classmethod
    def nova(cls):
        """nova client

        :rtype: novaclient.v2.client.Client
        """
        if not cls._nova:
            n_client = driver_module('nova')
            client = n_client.Client(
                version=CONF.root_cause_analysis_service.nova_version,
                session=cls._get_session_for_admin(),
            )
            cls._nova = client
        return cls._nova

    @classmethod
    def cinder(cls):
        """cinder client

        :rtype: cinderclient.v2.client.Client
        """
        if not cls._cinder:
            cin_client = driver_module('cinder')
            client = cin_client.Client(
                version=CONF.root_cause_analysis_service.cinder_version,
                session=cls._get_session_for_admin(),
            )
            cls._cinder = client
        return cls._cinder

    @classmethod
    def glance(cls):
        """glance client

        :rtype: glanceclient.v2.client.Client
        """
        if not cls._glance:

            glan_client = driver_module('glance')
            client = glan_client.Client(
                version=CONF.root_cause_analysis_service.glance_version,
                session=cls._get_session_for_admin(),
            )
            cls._glance = client
        return cls._glance

    @classmethod
    def neutron(cls):
        """neutron client

        :rtype: neutronclient.v2_0.client.Client
        """
        if not cls._neutron:
            ne_client = driver_module('neutron')
            client = ne_client.Client(
                session=cls._get_session_for_admin()
            )
            cls._neutron = client
        return cls._neutron

    @classmethod
    def heat(cls):
        """heat client

        :rtype: heatclient.v1.client.Client
        """
        if not cls._heat:
            he_client = driver_module('heat')
            client = he_client.Client(
                version=CONF.root_cause_analysis_service.heat_version,
                session=cls._get_session_for_admin()
            )
            cls._heat = client
        return cls._heat

    @classmethod
    def mistral(cls):
        """mistral client

        :rtype: mistralclient.v2.client.Client
        """
        if not cls._mistral:
            mi_client = driver_module('mistral')
            client = mi_client.Client(
                session=cls._get_session_for_admin(),
            )
            cls._mistral = client
        return cls._mistral

    @classmethod
    def aodh(cls):
        """aodh client

        :rtype: aodhclient.v2.client.Client
        """
        if not cls._aodh:
            ao_client = driver_module('aodh')
            client = ao_client.Client(
                CONF.root_cause_analysis_service.aodh_version,
                session=cls._get_session_for_admin())
            cls._aodh = client
        return cls._aodh

    @classmethod
    def keystone(cls):
        """keystone client

        :rtype: keystoneclient.v3.client.Client
        """
        if not cls._keystone:
            sess = cls._get_session_for_admin()
            cls._keystone = ks_client_v3.Client(session=sess)
        return cls._keystone

    @classmethod
    def gnocchi(cls):
        """gnocchi client

        :rtype: gnocchiclient.v1.client.Client
        """
        if not cls._gnocchi:
            gn_client = driver_module('gnocchi')
            client = gn_client.Client(
                session=cls._get_session_for_admin())
            cls._gnocchi = client
        return cls._gnocchi

    @classmethod
    def _get_session_for_admin(cls):
        admin_creds = common_creds.get_configured_admin_credentials()
        password = admin_creds.password
        username = admin_creds.username
        user_domain_id = admin_creds.user_domain_id
        project_name = admin_creds.project_name
        project_domain_id = admin_creds.project_domain_id

        return cls._get_session(username, password, user_domain_id,
                                project_name, project_domain_id)

    @classmethod
    def _get_session_for_user(cls):
        username = cls.creds.username
        password = cls.creds.password
        user_domain_id = cls.creds.user_domain_id
        project_name = cls.creds.project_name
        project_domain_id = cls.creds.project_domain_id

        return cls._get_session(username, password, user_domain_id,
                                project_name, project_domain_id)

    @classmethod
    def _get_session(cls, username, password, user_domain_id, project_name,
                     project_domain_id):
        loader = ka_loading.get_plugin_loader('password')
        auth_url = CONF.identity.uri_v3
        auth_plugin = loader.load_from_options(
            auth_url=auth_url,
            username=username, password=password, project_name=project_name,
            project_domain_id=project_domain_id,
            user_domain_id=user_domain_id)
        return ka_session.Session(auth=auth_plugin)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from vitrage_tempest_plugin.tests.api.topology.base import BaseTopologyTest
from vitrage_tempest_plugin.tests import utils

LOG = logging.getLogger(__name__)


class TestCinderVolume(BaseTopologyTest):
    NUM_INSTANCE = 3
    NUM_VOLUME = 1

    def tearDown(self):
        super(TestCinderVolume, self).tearDown()
        self._rollback_to_default()

    @utils.tempest_logger
    def test_volume(self):
        # Action
        self._create_entities(num_instances=self.NUM_INSTANCE,
                              num_volumes=self.NUM_VOLUME)

        # Calculate expected results
        api_graph = self.vitrage_client.topology.get(all_tenants=True)
        graph = self._create_graph_from_graph_dictionary(api_graph)
        entities = self._entities_validation_data(
            host_entities=1,
            host_edges=self.NUM_INSTANCE + 1,
            instance_entities=self.NUM_INSTANCE,
            instance_edges=2 * self.NUM_INSTANCE + self.NUM_VOLUME,
            volume_entities=self.NUM_VOLUME,
            volume_edges=self.NUM_VOLUME)
        num_entities = self.num_default_entities + self.NUM_VOLUME + \
            2 * self.NUM_INSTANCE + self.num_default_ports + \
            self.num_default_networks
        num_edges = self.num_default_edges + 3 * self.NUM_INSTANCE + \
            self.NUM_VOLUME + self.num_default_ports

        # Test Assertions
        self._validate_graph_correctness(graph,
                                         num_entities,
                                         num_edges,
                                         entities)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time

from heatclient.common import http
from heatclient.common import template_utils

from vitrage_tempest_plugin.tests.common.tempest_clients import TempestClients
from vitrage_tempest_plugin.tests.utils import wait_for_status


def create_stacks(num_stacks, nested, template_file, image):
    tpl_files, template = template_utils.process_template_path(
        template_file,
        object_request=http.authenticated_fetcher(TempestClients.heat()))

    for i in range(num_stacks):
        stack_name = 'stack_%s' % i + ('_nested' if nested else '')
        TempestClients.heat().stacks.create(stack_name=stack_name,
                                            template=template,
                                            files=tpl_files,
                                            parameters={'image': image})
        wait_for_status(45,
                        _check_num_stacks,
                        num_stacks=num_stacks,
                        state='CREATE_COMPLETE')
        time.sleep(2)


def delete_all_stacks():
    stacks = TempestClients.heat().stacks.list()
    for stack in stacks:
        try:
            TempestClients.heat().stacks.delete(stack.to_dict()['id'])
        except Exception:
            pass

    wait_for_status(30, _check_num_stacks, num_stacks=0)
    time.sleep(4)


def _check_num_stacks(num_stacks, state=''):
    stacks_list = \
        [stack.to_dict() for stack in TempestClients.heat().stacks.list()
         if 'FAILED' not in stack.to_dict()['stack_status']]
    if len(stacks_list) != num_stacks:
        return False

    return all(stack['stack_status'].upper() == state.upper()
               for stack in stacks_list)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from datetime import datetime
import os

from oslo_log import log as logging

from vitrage_tempest_plugin.tests.common.constants import NOVA_HOST_DATASOURCE
from vitrage_tempest_plugin.tests.common.constants import \
    NOVA_INSTANCE_DATASOURCE
from vitrage_tempest_plugin.tests.common.constants import TemplateStatus
from vitrage_tempest_plugin.tests.common.constants import TemplateTypes
from vitrage_tempest_plugin.tests.common import general_utils as g_utils
from vitrage_tempest_plugin.tests.common.tempest_clients import TempestClients
from vitrage_tempest_plugin.tests.utils import wait_for_status

LOG = logging.getLogger(__name__)

DOWN = 'down'
UP = 'up'


def generate_fake_host_alarm(hostname, event_type, enabled=True):
    details = {
        'hostname': hostname,
        'source': 'fake_tempest_monitor',
        'cause': 'another alarm',
        'severity': 'critical',
        'status': DOWN if enabled else UP,
        'monitor_id': 'fake tempest monitor id',
        'monitor_event_id': '111',
    }
    event_time = datetime.now()
    event_time_iso = event_time.isoformat()
    TempestClients.vitrage().event.post(event_time_iso, event_type, details)


def get_first_host(**kwargs):
    hosts = TempestClients.vitrage().resource.list(
        NOVA_HOST_DATASOURCE, all_tenants=True)
    return g_utils.first_match(hosts, **kwargs)


def get_first_instance(**kwargs):
    instances = TempestClients.vitrage().resource.list(
        NOVA_INSTANCE_DATASOURCE, all_tenants=True)
    return g_utils.first_match(instances, **kwargs)


def add_template(filename='',
                 folder='templates/api',
                 template_type=TemplateTypes.STANDARD,
                 status=TemplateStatus.ACTIVE):
    full_path = g_utils.tempest_resources_dir() + '/' + folder + '/' + filename
    t = TempestClients.vitrage().template.add(full_path, template_type)
    if t and t[0]:
        wait_for_status(
            100,
            get_first_template,
            uuid=t[0]['uuid'], status=status)
        return t[0]
    return None


def get_first_template(**kwargs):
    templates = TempestClients.vitrage().template.list()
    return g_utils.first_match(templates, **kwargs)


def delete_template(uuid=None, **kwargs):
    if not uuid:
        template = get_first_template(**kwargs)
        if template:
            uuid = template['uuid']
        else:
            return
    TempestClients.vitrage().template.delete(uuid)
    wait_for_status(
        100,
        lambda _id: True if not get_first_template(uuid=_id) else False,
        _id=uuid)


def delete_template_with_name(name):
    TempestClients.vitrage().template.delete(name)
    wait_for_status(
        100,
        lambda name: True if not get_first_template(name=name) else False,
        name=name)


def restart_graph():
    os.system("sudo service devstack@vitrage-graph restart")


def stop_graph():
    os.system("sudo service devstack@vitrage-graph stop")



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import testtools
from testtools import matchers
import yaml

from vitrage_tempest_plugin.tests.api.templates.base import BaseTemplateTest
from vitrage_tempest_plugin.tests.base import IsNotEmpty
from vitrage_tempest_plugin.tests.common.constants import TemplateStatus
from vitrage_tempest_plugin.tests.common.constants import TemplateTypes as \
    TTypes
from vitrage_tempest_plugin.tests.common import general_utils as g_utils
from vitrage_tempest_plugin.tests.common import vitrage_utils as v_utils

import vitrage_tempest_plugin.tests.utils as utils
from vitrageclient.exceptions import ClientException

LOG = logging.getLogger(__name__)

STANDARD_TEMPLATE = 'host_high_memory_usage_scenarios.yaml'
EQUIVALENCE_TEMPLATE = 'basic_equivalence_templates.yaml'
DEFINITION_TEMPLATE = 'basic_def_template.yaml'
STANDARD_ERROR = 'corrupted_template.yaml'

FAKE_UUID = 'ade68276-0fe9-42cd-9ec2-e7f20470a771'


class TestValidate(BaseTemplateTest):
    """Template test class for Vitrage API tests."""

    @classmethod
    def setUpClass(cls):
        super(TestValidate, cls).setUpClass()
        cls._template = v_utils.add_template(STANDARD_TEMPLATE)

    @classmethod
    def tearDownClass(cls):
        super(TestValidate, cls).tearDownClass()
        if cls._template is not None:
            v_utils.delete_template(cls._template['uuid'])

    def test_templates_list(self):
        """template_list test

        There test validate correctness of template list,
        equals templates files existence with default folder
        and between cli via api ...
        """
        api_template_list = self.vitrage_client.template.list()
        cli_template_list = utils.run_vitrage_command(
            'vitrage template list')

        self._compare_template_lists(api_template_list, cli_template_list)

    def test_templates_validate_non_exist_template(self):
        """templates_validate test

        There negative test - validate error message
         in case of non-exist template file
        """
        path = self.TEST_PATH + self.NON_EXIST_FILE
        self.assertRaises(IOError,
                          self.vitrage_client.template.validate,
                          path=path)

    def test_templates_validate_corrupted_templates(self):
        """templates_validate test

        There negative test - validate correctness of error
        message in case of corrupted template file
        """
        path = self.TEST_PATH + self.ERROR_FILE
        validation = self.vitrage_client.template.validate(path=path)
        self.assertThat(validation['results'], matchers.HasLength(1))
        self._assert_validate_result(validation, path,
                                     negative=True, status_code=3)

    def test_templates_validate_correct_template(self):
        """templates_validate test

        There test validate correctness of template file
        """
        path = self.TEST_PATH + self.OK_FILE
        validation = self.vitrage_client.template.validate(path=path)
        self.assertThat(validation['results'], matchers.HasLength(1))
        self._assert_validate_result(validation, path)


@testtools.skip("skip for now")
class TemplatesDBTest(BaseTemplateTest):
    """Template DB test class for vitrage API tests"""

    def test_template_add(self):
        """template add test

        test standard , definition and equivalence templates
        """
        # TODO(ikinory): add folder of templates
        # Add standard ,equivalence and definition templates
        self._add_templates()
        # assert standard template
        db_row = v_utils.get_first_template(
            name='host_high_memory_usage_scenarios', type=TTypes.STANDARD)
        self.assertEqual(db_row['name'],
                         'host_high_memory_usage_scenarios',
                         'standard template not found in list')

        # assert equivalence template
        db_row = v_utils.get_first_template(
            name='entity equivalence example',
            type=TTypes.EQUIVALENCE)
        self.assertEqual(db_row['name'],
                         'entity equivalence example',
                         'equivalence template not found in list')

        # assert definition template
        db_row = v_utils.get_first_template(
            name='basic_def_template',
            type=TTypes.DEFINITION,
            status=TemplateStatus.ACTIVE)

        self.assertEqual(db_row['name'],
                         'basic_def_template',
                         'definition template not found in list')

        # assert corrupted template - validate failed
        db_row = v_utils.get_first_template(
            name='corrupted_template',
            type=TTypes.STANDARD,
            status=TemplateStatus.ERROR)
        self.assertIsNotNone(
            db_row,
            'corrupted template template presented in list')

    def test_template_delete(self):
        self._add_delete_template()

    def test_template_delete_with_name(self):
        self._add_delete_template_with_name()

    def test_template_dont_show_deleted_template(self):

        uuid = self._add_delete_template()
        self.assertRaises(ClientException,
                          self.vitrage_client.template.show,
                          uuid)

    def test_compare_cli_to_api(self):
        """Compare between api template list

        to cli template list
        compares each template in list
        """
        # Add standard ,equivalence and definition templates
        self._add_templates()
        cli_templates_list = utils.run_vitrage_command(
            "vitrage template list")
        api_templates_list = self.vitrage_client.template.list()

        self.assertThat(api_templates_list, IsNotEmpty(),
                        'The template list taken from api is empty')
        self.assertIsNotNone(cli_templates_list,
                             'The template list taken from cli is empty')
        self._validate_templates_list_length(api_templates_list,
                                             cli_templates_list)
        self._validate_passed_templates_length(api_templates_list,
                                               cli_templates_list)
        self._compare_each_template_in_list(api_templates_list,
                                            cli_templates_list)

    def test_template_show(self):
        """Compare template content from file to DB"""
        # add standard template
        template_path = \
            g_utils.tempest_resources_dir() + '/templates/api/'\
            + STANDARD_TEMPLATE
        v_utils.add_template(STANDARD_TEMPLATE,
                             template_type=TTypes.STANDARD)
        db_row = v_utils.get_first_template(
            name='host_high_memory_usage_scenarios',
            type=TTypes.STANDARD,
            status=TemplateStatus.ACTIVE)
        payload_from_db = self.vitrage_client.template.show(db_row['uuid'])
        with open(template_path, 'r') as stream:
            payload_from_file = yaml.load(stream, Loader=yaml.BaseLoader)
        self.assert_dict_equal(payload_from_file, payload_from_db,
                               "Template content doesn't match")
        v_utils.delete_template(db_row['uuid'])

    def test_template_show_with_name(self):
        """Compare template content from file to DB"""
        # add standard template
        template_path = \
            g_utils.tempest_resources_dir() + '/templates/api/' \
            + STANDARD_TEMPLATE
        v_utils.add_template(STANDARD_TEMPLATE,
                             template_type=TTypes.STANDARD)
        name = 'host_high_memory_usage_scenarios'
        db_row = v_utils.get_first_template(
            name=name,
            type=TTypes.STANDARD,
            status=TemplateStatus.ACTIVE)
        payload_from_db = self.vitrage_client.template.show(name)
        with open(template_path, 'r') as stream:
            payload_from_file = yaml.load(stream, Loader=yaml.BaseLoader)
        self.assert_dict_equal(payload_from_file, payload_from_db,
                               "Template content doesn't match")
        v_utils.delete_template(db_row['uuid'])

    @staticmethod
    def _add_templates():
        v_utils.add_template(STANDARD_TEMPLATE,
                             template_type=TTypes.STANDARD)
        v_utils.add_template(EQUIVALENCE_TEMPLATE,
                             template_type=TTypes.EQUIVALENCE)
        v_utils.add_template(DEFINITION_TEMPLATE,
                             template_type=TTypes.DEFINITION)
        v_utils.add_template(STANDARD_ERROR,
                             template_type=TTypes.STANDARD,
                             status=TemplateStatus.ERROR)
        return ['host_high_memory_usage_scenarios',
                'entity equivalence example',
                'basic_def_template',
                'corrupted_template']

    def _add_delete_template(self):
        """A helper function:

            Adds and deletes a template.
            Returns its uuid.
        """

        # add a template
        v_utils.add_template(STANDARD_TEMPLATE, template_type=TTypes.STANDARD)
        db_row = v_utils.get_first_template(
            name='host_high_memory_usage_scenarios',
            type=TTypes.STANDARD,
            status=TemplateStatus.ACTIVE)
        self.assertIsNotNone(db_row,
                             'Template should appear in templates list')

        # delete it
        uuid = db_row['uuid']
        v_utils.delete_template(uuid)
        db_row = v_utils.get_first_template(
            name='host_high_memory_usage_scenarios',
            type=TTypes.STANDARD)
        self.assertIsNone(db_row, 'Template should not appear in list')

        return uuid

    def _add_delete_template_with_name(self):
        """A helper function:

            Adds and deletes a template.
            Returns its name.
        """

        # add a template
        v_utils.add_template(STANDARD_TEMPLATE, template_type=TTypes.STANDARD)
        db_row = v_utils.get_first_template(
            name='host_high_memory_usage_scenarios',
            type=TTypes.STANDARD,
            status=TemplateStatus.ACTIVE)
        self.assertIsNotNone(db_row,
                             'Template should appear in templates list')

        # delete it
        name = db_row['name']
        v_utils.delete_template_with_name(name)
        db_row = v_utils.get_first_template(
            name='host_high_memory_usage_scenarios',
            type=TTypes.STANDARD)
        self.assertIsNone(db_row, 'Template should not appear in list')

        return name
